// Copyright (c) 2008  Future Technology Devices International Ltd.
//
// Module Name
//
//     FT2232HJTAGUnit.pas
//
// Abstract:
//
//    Delphi program, to test the FTCJTAG.DLL. The FTCJTAG.DLL is used to control the FT2232H dual hi-speed device
//    and the FT4232H quad hi-speed device to simulate the Joint Test Action Group(JTAG) synchronous protocol to
//    communicate with I2C devices connected to FT2232H dual hi-speed devices and FT4232H quad hi-speed devices.
//
// Revision History:
//
//   16/09/08    Kirke Ashton    Created.
//
unit FT2232HJTAGUnit;

interface

uses
  Windows;

type FTC_STATUS = Integer;

const MAX_NUM_DEVICE_NAME_CHARS = 100;

type TDeviceName = array [0..(MAX_NUM_DEVICE_NAME_CHARS - 1)] of Char;
type PDeviceName = ^TDeviceName;

const MAX_NUM_CHANNEL_CHARS = 5;

type TChannel = array [0..(MAX_NUM_CHANNEL_CHARS - 1)] of Char;
type PChannel = ^TChannel;

type HiSpeedDeviceTypes = (FT2232H_DEVICE_TYPE = 1, FT4232H_DEVICE_TYPE = 2);

type FtcInputOutputPins = record
  Pin1InputOutputState: LongBool;
  Pin1LowHighState: LongBool;
  Pin2InputOutputState: LongBool;
  Pin2LowHighState: LongBool;
  Pin3InputOutputState: LongBool;
  Pin3LowHighState: LongBool;
  Pin4InputOutputState: LongBool;
  Pin4LowHighState: LongBool;
end;

type PFtcInputOutputPins = ^FtcInputOutputPins;

type FthInputOutputPins = record
  Pin1InputOutputState: LongBool;
  Pin1LowHighState: LongBool;
  Pin2InputOutputState: LongBool;
  Pin2LowHighState: LongBool;
  Pin3InputOutputState: LongBool;
  Pin3LowHighState: LongBool;
  Pin4InputOutputState: LongBool;
  Pin4LowHighState: LongBool;
  Pin5InputOutputState: LongBool;
  Pin5LowHighState: LongBool;
  Pin6InputOutputState: LongBool;
  Pin6LowHighState: LongBool;
  Pin7InputOutputState: LongBool;
  Pin7LowHighState: LongBool;
  Pin8InputOutputState: LongBool;
  Pin8LowHighState: LongBool;
end;

type PFthInputOutputPins = ^FthInputOutputPins;

type FtcLowHighPins = record
  Pin1LowHighState: LongBool;
  Pin2LowHighState: LongBool;
  Pin3LowHighState: LongBool;
  Pin4LowHighState: LongBool;
end;

type FthLowHighPins = record
  Pin1LowHighState: LongBool;
  Pin2LowHighState: LongBool;
  Pin3LowHighState: LongBool;
  Pin4LowHighState: LongBool;
  Pin5LowHighState: LongBool;
  Pin6LowHighState: LongBool;
  Pin7LowHighState: LongBool;
  Pin8LowHighState: LongBool;
end;

type PFthLowHighPins = ^FthLowHighPins;

type PFtcLowHighPins = ^FtcLowHighPins;

const MAX_WRITE_DATA_BYTES_BUFFER_SIZE = 65535;    // 64k bytes

type WriteDataByteBuffer = array[0..(MAX_WRITE_DATA_BYTES_BUFFER_SIZE - 1)] of Byte;
type PWriteDataByteBuffer = ^WriteDataByteBuffer;

const MAX_READ_DATA_BYTES_BUFFER_SIZE = 65535;    // 64k bytes

type ReadDataByteBuffer = array[0..(MAX_READ_DATA_BYTES_BUFFER_SIZE - 1)] of Byte;
type PReadDataByteBuffer = ^ReadDataByteBuffer;

const MAX_READ_CMDS_DATA_BYTES_BUFFER_SIZE = 131071;  // 128K bytes 

type ReadCmdSequenceDataByteBuffer = array[0..(MAX_READ_CMDS_DATA_BYTES_BUFFER_SIZE - 1)] of Byte;
type PReadCmdSequenceDataByteBuffer = ^ReadCmdSequenceDataByteBuffer;

const MAX_NUM_DLL_VERSION_CHARS = 10;

type TDllVersion = array [0..(MAX_NUM_DLL_VERSION_CHARS - 1)] of Char;
type PDllVersion = ^TDllVersion;

const MAX_NUM_ERROR_MESSAGE_CHARS = 100;

type TErrorMessage = array [0..(MAX_NUM_ERROR_MESSAGE_CHARS - 1)] of Char;
type PErrorMessage = ^TErrorMessage;

type TFT2232HJTAG = class
 public
  constructor create;
  function  GetNumHiSpeedDevices(pNumHiSpeedDevices: PDword): FTC_STATUS;
  function  GetHiSpeedDeviceNameLocIDChannel(DeviceNameIndex: Dword; pDeviceNameBuffer: PDeviceName; DeviceNameBufferSize: Dword; pLocationID: PDword; pChannelBuffer: PChannel; ChannelBufferSize: Dword; pHiSpeedDeviceType: PDword): FTC_STATUS;
  function  OpenHiSpeedDevice(DeviceName: String; LocationID: Dword; Channel: String; pFtHandle: PDword): FTC_STATUS;
  function  GetHiSpeedDeviceType(fthandle: Dword; pHiSpeedDeviceType: PDword): FTC_STATUS;
  function  Close(fthandle: Dword): FTC_STATUS;
  function  InitDevice(fthandle: Dword; ClockFrequencyValue: Dword): FTC_STATUS;
  function  GetClock(ClockDivisor: Dword; pClockFrequencyHz: PDword): FTC_STATUS;
  function  SetClock(fthandle: Dword; ClockDivisor: Dword; pClockFrequencyHz: PDword): FTC_STATUS;
  function  SetLoopback(fthandle: Dword; bLoopbackState: LongBool): FTC_STATUS;
  function  SetHiSpeedDeviceGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool; LowInputOutputPins: PFtcInputOutputPins; bControlHighInputOutputPins: LongBool; HighInputOutputPins: PFthInputOutputPins): FTC_STATUS;
  function  GetHiSpeedDeviceGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool; LowPinsInputData: PFtcLowHighPins; bControlHighInputOutputPins: LongBool; HighPinsInputData: PFthLowHighPins): FTC_STATUS;
  //function  SetGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool; LowInputOutputPins: PFtcInputOutputPins; bControlHighInputOutputPins: LongBool; HighInputOutputPins: PFtcInputOutputPins): FTC_STATUS;
  //function  GetGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool; LowPinsInputData: PFtcLowHighPins; bControlHighInputOutputPins: LongBool; HighPinsInputData: PFtcLowHighPins): FTC_STATUS;
  function  Write(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToWrite: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS;
  function  Read(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToRead: Dword; ReadDataBuffer: PReadDataByteBuffer; pNumBytesReturned: PDword; TapControllerState: Dword): FTC_STATUS;
  function  WriteRead(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToWriteRead: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; ReadDataBuffer: PReadDataByteBuffer; pNumBytesReturned: PDword; TapControllerState: Dword): FTC_STATUS;
  function  GenerateClockPulses(fthandle: Dword; NumClockPulses: Dword): FTC_STATUS;  
  //function  ClearCmdSequence(): FTC_STATUS;
  //function  AddWriteCmd(bInstructionTestData: LongBool; NumBitsToWrite: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS;
  //function  AddReadCmd(bInstructionTestData: LongBool; NumBitsToRead: Dword; TapControllerState: Dword): FTC_STATUS;
  //function  AddWriteReadCmd(bInstructionTestData: LongBool; NumBitsToWriteRead: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS;
  function  ClearDeviceCmdSequence(fthandle: Dword): FTC_STATUS;
  function  AddDeviceWriteCmd(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToWrite: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS;
  function  AddDeviceReadCmd(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToRead: Dword; TapControllerState: Dword): FTC_STATUS;
  function  AddDeviceWriteReadCmd(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToWriteRead: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS;
  function  ExecuteCmdSequence(fthandle: Dword; ReadCmdSequenceDataByteBuffer: PReadCmdSequenceDataByteBuffer; pNumBytesReturned: PDword): FTC_STATUS;
  function  GetDllVersion(DllVersionBuffer: pDllVersion; BufferSize: Dword): FTC_STATUS;
  function  GetErrorCodeString(Language: String; StatusCode: FTC_STATUS; pErrorMessageBuffer: PErrorMessage; BufferSize: Dword): FTC_STATUS;
end;


const
  FT2232CJTAG_DLL_Name = 'ftcjtag.dll';

  TEST_LOGIC_STATE = 1;
  RUN_TEST_IDLE_STATE = 2;
  PAUSE_TEST_DATA_REGISTER_STATE = 3;
  PAUSE_INSTRUCTION_REGISTER_STATE = 4;
  SHIFT_TEST_DATA_REGISTER_STATE = 5;
  SHIFT_INSTRUCTION_REGISTER_STATE = 6;

  FTC_SUCCESS = 0;
  FTC_NO_COMMAND_SEQUENCE = 39;

  ENGLISH = 'EN';

  
implementation

function JTAG_GetNumHiSpeedDevices(pNumHiSpeedDevices: PDword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_GetNumHiSpeedDevices';
function JTAG_GetHiSpeedDeviceNameLocIDChannel(DeviceNameIndex: Dword; pDeviceNameBuffer: PDeviceName; DeviceNameBufferSize: Dword; pLocationID: PDword; pChannelBuffer: PChannel; ChannelBufferSize: Dword; pHiSpeedDeviceType: PDword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_GetHiSpeedDeviceNameLocIDChannel';
function JTAG_OpenHiSpeedDevice(DeviceName: String; LocationID: Dword; Channel: String; pFtHandle: PDword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_OpenHiSpeedDevice';
function JTAG_GetHiSpeedDeviceType(fthandle: Dword; pHiSpeedDeviceType: PDword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_GetHiSpeedDeviceType';
function JTAG_Close(fthandle: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_Close';
function JTAG_InitDevice(fthandle: Dword; ClockFrequencyValue: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_InitDevice';
function JTAG_GetClock(ClockDivisor: Dword; pClockFrequencyHz: PDword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_GetClock';
function JTAG_SetClock(fthandle: Dword; ClockDivisor: Dword; pClockFrequencyHz: PDword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_SetClock';
function JTAG_SetLoopback(fthandle: Dword; bLoopbackState: LongBool): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_SetLoopback';
function JTAG_SetHiSpeedDeviceGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool; LowInputOutputPins: PFtcInputOutputPins; bControlHighInputOutputPins: LongBool; HighInputOutputPins: PFthInputOutputPins): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_SetHiSpeedDeviceGPIOs';
function JTAG_GetHiSpeedDeviceGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool; LowPinsInputData: PFtcLowHighPins; bControlHighInputOutputPins: LongBool; HighPinsInputData: PFthLowHighPins): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_GetHiSpeedDeviceGPIOs';
//function JTAG_SetGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool; LowInputOutputPins: PFtcInputOutputPins; bControlHighInputOutputPins: LongBool; HighInputOutputPins: PFtcInputOutputPins): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_SetGPIOs';
//function JTAG_GetGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool; LowPinsInputData: PFtcLowHighPins; bControlHighInputOutputPins: LongBool; HighPinsInputData: PFtcLowHighPins): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_GetGPIOs';
function JTAG_Write(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToWrite: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_Write';
function JTAG_Read(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToRead: Dword; ReadDataBuffer: PReadDataByteBuffer; pNumBytesReturned: PDword; TapControllerState: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_Read';
function JTAG_WriteRead(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToWriteRead: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; ReadDataBuffer: PReadDataByteBuffer; pNumBytesReturned: PDword; TapControllerState: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_WriteRead';
function JTAG_GenerateClockPulsesHiSpeedDevice(fthandle: Dword; bPulseClockTimesEightFactor: LongBool; NumClockPulses: Dword; bControlLowInputOutputPin: LongBool; bStopClockPulsesState: LongBool): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_GenerateClockPulsesHiSpeedDevice';
//function JTAG_ClearCmdSequence(): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_ClearCmdSequence';
//function JTAG_AddWriteCmd(bInstructionTestData: LongBool; NumBitsToWrite: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_AddWriteCmd';
//function JTAG_AddReadCmd(bInstructionTestData: LongBool; NumBitsToRead: Dword; TapControllerState: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_AddReadCmd';
//function JTAG_AddWriteReadCmd(bInstructionTestData: LongBool; NumBitsToWriteRead: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_AddWriteReadCmd';
function JTAG_ClearDeviceCmdSequence(fthandle: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_ClearDeviceCmdSequence';
function JTAG_AddDeviceWriteCmd(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToWrite: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_AddDeviceWriteCmd';
function JTAG_AddDeviceReadCmd(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToRead: Dword; TapControllerState: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_AddDeviceReadCmd';
function JTAG_AddDeviceWriteReadCmd(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToWriteRead: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_AddDeviceWriteReadCmd';
function JTAG_ExecuteCmdSequence(fthandle: Dword; ReadCmdSequenceDataByteBuffer: PReadCmdSequenceDataByteBuffer; pNumBytesReturned: PDword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_ExecuteCmdSequence';
function JTAG_GetDllVersion(DllVersionBuffer: pDllVersion; BufferSize: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_GetDllVersion';
function JTAG_GetErrorCodeString(Language: PChar; StatusCode: FTC_STATUS; ErrorMessageBuffer: PErrorMessage; BufferSize: Dword): FTC_STATUS; stdcall ; External FT2232CJTAG_DLL_Name name 'JTAG_GetErrorCodeString';


constructor TFT2232HJTAG.create;
begin
end;

function  TFT2232HJTAG.GetNumHiSpeedDevices(pNumHiSpeedDevices: PDword): FTC_STATUS;
begin
  Result := JTAG_GetNumHiSpeedDevices(pNumHiSpeedDevices);
end;

function  TFT2232HJTAG.GetHiSpeedDeviceNameLocIDChannel(DeviceNameIndex: Dword; pDeviceNameBuffer: PDeviceName; DeviceNameBufferSize: Dword; pLocationID: PDword; pChannelBuffer: PChannel; ChannelBufferSize: Dword; pHiSpeedDeviceType: PDword): FTC_STATUS;
begin
  Result := JTAG_GetHiSpeedDeviceNameLocIDChannel(DeviceNameIndex, pDeviceNameBuffer, DeviceNameBufferSize, pLocationID, pChannelBuffer, ChannelBufferSize, pHiSpeedDeviceType);
end;

function  TFT2232HJTAG.OpenHiSpeedDevice(DeviceName: String; LocationID: Dword; Channel: String; pFtHandle: PDword): FTC_STATUS;
begin
  Result := JTAG_OpenHiSpeedDevice(DeviceName, LocationID, Channel, pFtHandle);
end;

function  TFT2232HJTAG.GetHiSpeedDeviceType(fthandle: Dword; pHiSpeedDeviceType: PDword): FTC_STATUS;
begin
  Result := JTAG_GetHiSpeedDeviceType(fthandle, pHiSpeedDeviceType);
end;

function  TFT2232HJTAG.Close(fthandle: Dword): FTC_STATUS;
begin
  Result := JTAG_Close(fthandle);
end;

function  TFT2232HJTAG.InitDevice(fthandle: Dword; ClockFrequencyValue: Dword): FTC_STATUS;
begin
  Result := JTAG_InitDevice(fthandle, ClockFrequencyValue);
end;

function  TFT2232HJTAG.GetClock(ClockDivisor: Dword; pClockFrequencyHz: PDword): FTC_STATUS;
begin
  Result := JTAG_GetClock(ClockDivisor, pClockFrequencyHz);
end;

function  TFT2232HJTAG.SetClock(fthandle: Dword; ClockDivisor: Dword; pClockFrequencyHz: PDword): FTC_STATUS;
begin
  Result :=  JTAG_SetClock(fthandle, ClockDivisor, pClockFrequencyHz);
end;

function  TFT2232HJTAG.SetLoopback(fthandle: Dword; bLoopbackState: LongBool): FTC_STATUS;
begin
  Result := JTAG_SetLoopback(fthandle, bLoopbackState);
end;

function  TFT2232HJTAG.SetHiSpeedDeviceGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool;
                                             LowInputOutputPins: PFtcInputOutputPins;
                                             bControlHighInputOutputPins: LongBool;
                                             HighInputOutputPins: PFthInputOutputPins): FTC_STATUS;

begin
  Result := JTAG_SetHiSpeedDeviceGPIOs(fthandle, bControlLowInputOutputPins, LowInputOutputPins, bControlHighInputOutputPins, HighInputOutputPins);
end;

function  TFT2232HJTAG.GetHiSpeedDeviceGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool;
                                             LowPinsInputData: PFtcLowHighPins;
                                             bControlHighInputOutputPins: LongBool;
                                             HighPinsInputData: PFthLowHighPins): FTC_STATUS;
begin
  Result := JTAG_GetHiSpeedDeviceGPIOs(fthandle, bControlLowInputOutputPins, LowPinsInputData, bControlHighInputOutputPins, HighPinsInputData);
end;

function  TFT2232HJTAG.Write(fthandle: Dword; bInstructionTestData: LongBool;
                             NumBitsToWrite: Dword; WriteDataBuffer: PWriteDataByteBuffer;
                             NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS;
begin
  Result := JTAG_Write(fthandle, bInstructionTestData, NumBitsToWrite, WriteDataBuffer, NumBytesToWrite, TapControllerState);
end;

function  TFT2232HJTAG.Read(fthandle: Dword; bInstructionTestData: LongBool;
                            NumBitsToRead: Dword; ReadDataBuffer: PReadDataByteBuffer;
                            pNumBytesReturned: PDword; TapControllerState: Dword): FTC_STATUS;
begin
  Result := JTAG_Read(fthandle, bInstructionTestData, NumBitsToRead, ReadDataBuffer, pNumBytesReturned, TapControllerState);
end;

function  TFT2232HJTAG.WriteRead(fthandle: Dword; bInstructionTestData: LongBool;
                                 NumBitsToWriteRead: Dword; WriteDataBuffer: PWriteDataByteBuffer;
                                 NumBytesToWrite: Dword; ReadDataBuffer: PReadDataByteBuffer;
                                 pNumBytesReturned: PDword; TapControllerState: Dword): FTC_STATUS;
begin
  Result := JTAG_WriteRead(fthandle, bInstructionTestData, NumBitsToWriteRead, WriteDataBuffer, NumBytesToWrite, ReadDataBuffer, pNumBytesReturned, TapControllerState);
end;

function  TFT2232HJTAG.GenerateClockPulses(fthandle: Dword; NumClockPulses: Dword): FTC_STATUS;
begin
  Result := JTAG_GenerateClockPulsesHiSpeedDevice(fthandle, True, NumClockPulses, False, False);
end;

//function  TFT2232HJTAG.ClearCmdSequence(): FTC_STATUS;
//begin
//  Result := JTAG_ClearCmdSequence();
//end;

//function  TFT2232HJTAG.AddWriteCmd(bInstructionTestData: LongBool; NumBitsToWrite: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS;
//begin
//  Result := JTAG_AddWriteCmd(bInstructionTestData, NumBitsToWrite, WriteDataBuffer,
//                             NumBytesToWrite, TapControllerState);
//end;

//function  TFT2232HJTAG.AddReadCmd(bInstructionTestData: LongBool; NumBitsToRead: Dword; TapControllerState: Dword): FTC_STATUS;
//begin
//  Result := JTAG_AddReadCmd(bInstructionTestData, NumBitsToRead, TapControllerState);
//end;

//function  TFT2232HJTAG.AddWriteReadCmd(bInstructionTestData: LongBool; NumBitsToWriteRead: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS;
//begin
//  Result := JTAG_AddWriteReadCmd(bInstructionTestData, NumBitsToWriteRead, WriteDataBuffer,
//                                 NumBytesToWrite, TapControllerState);
//end;

function  TFT2232HJTAG.ClearDeviceCmdSequence(fthandle: Dword): FTC_STATUS;
begin
  Result := JTAG_ClearDeviceCmdSequence(fthandle);
end;

function  TFT2232HJTAG.AddDeviceWriteCmd(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToWrite: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS;
begin
  Result := JTAG_AddDeviceWriteCmd(fthandle, bInstructionTestData, NumBitsToWrite, WriteDataBuffer,
                                   NumBytesToWrite, TapControllerState);
end;

function  TFT2232HJTAG.AddDeviceReadCmd(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToRead: Dword; TapControllerState: Dword): FTC_STATUS;
begin
  Result := JTAG_AddDeviceReadCmd(fthandle, bInstructionTestData, NumBitsToRead, TapControllerState);
end;

function  TFT2232HJTAG.AddDeviceWriteReadCmd(fthandle: Dword; bInstructionTestData: LongBool; NumBitsToWriteRead: Dword; WriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; TapControllerState: Dword): FTC_STATUS;
begin
  Result := JTAG_AddDeviceWriteReadCmd(fthandle, bInstructionTestData, NumBitsToWriteRead, WriteDataBuffer,
                                       NumBytesToWrite, TapControllerState);
end;

function  TFT2232HJTAG.ExecuteCmdSequence(fthandle: Dword; ReadCmdSequenceDataByteBuffer: PReadCmdSequenceDataByteBuffer; pNumBytesReturned: PDword): FTC_STATUS;
begin
  Result := JTAG_ExecuteCmdSequence(fthandle, ReadCmdSequenceDataByteBuffer, pNumBytesReturned);
end;

function  TFT2232HJTAG.GetDllVersion(DllVersionBuffer: pDllVersion; BufferSize: Dword): FTC_STATUS;
begin
  Result := JTAG_GetDllVersion(DllVersionBuffer, BufferSize);
end;

function  TFT2232HJTAG.GetErrorCodeString(Language: String; StatusCode: FTC_STATUS; pErrorMessageBuffer: PErrorMessage; BufferSize: Dword): FTC_STATUS;
begin
  Result := JTAG_GetErrorCodeString(PChar(Language), StatusCode, pErrorMessageBuffer, BufferSize);
end;

end.
